#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <zephyr/kernel.h>
#include <zephyr/drivers/uart.h>


#define UART_DEV DEVICE_DT_GET(DT_NODELABEL(uart1))
#define MAX_FRAME_SIZE 100

static uint8_t rx_buf[MAX_FRAME_SIZE];

static void uart_cb(const struct device *dev, struct uart_event *evt, void *user_data) {
    switch (evt->type) {
        case UART_RX_RDY:
            printk("UART ready!");
            break;

        case UART_RX_DISABLED:
            // RX buffer got full or RX stopped, re-enable to continue receiving
            uart_rx_enable(UART_DEV, rx_buf, sizeof(rx_buf), SYS_FOREVER_MS);
            break;

        case UART_RX_STOPPED:
            printk("UART RX stopped unexpectedly!");
            uart_rx_enable(UART_DEV, rx_buf, sizeof(rx_buf), SYS_FOREVER_MS);
            break;

        default:
            break;
    }
}

int mcu_interconnect_setup(void) {
    printk("Setting up MCU interconnect...");

    if (!device_is_ready(UART_DEV)) {
        printk("Device not ready for MCU interconnect");
        return -1;
    }
    
	printk("Setting up UART callback...");
    uart_callback_set(UART_DEV, uart_cb, NULL);

    printk("Enabling UART transmission...");
    uart_rx_enable(UART_DEV, rx_buf, sizeof(rx_buf), SYS_FOREVER_MS);

    return 0;
}

int main(void)
{
	mcu_interconnect_setup();

	int x = 0;
	uart_tx(UART_DEV, (const uint8_t *)&x, sizeof(int), SYS_FOREVER_MS);

	printk("Done!");

	return 0;
}
